@@ -1,15 +1,21 @@
+import inspect
 import itertools
 import math
-import numpy as np
-from scipy import ndimage as ndi
 from collections import OrderedDict
 from collections.abc import Iterable
-from ..exposure import histogram
-from .._shared.utils import check_nD, warn
-from ..transform import integral_image
-from ..util import crop, dtype_limits
-from ..filters._multiotsu import _find_threshold_multiotsu
 
+import numpy as np
+from scipy import ndimage as ndi
+
+from .._shared.filters import gaussian
+from .._shared.utils import _supported_float_type, warn
+from .._shared.version_requirements import require
+from ..exposure import histogram
+from ..filters._multiotsu import (_get_multiotsu_thresh_indices,
+                                  _get_multiotsu_thresh_indices_lut)
+from ..transform import integral_image
+from ..util import dtype_limits
+from ._sparse import _correlate_sparse, _validate_window_size
 
 __all__ = ['try_all_threshold',
            'threshold_otsu',
@@ -50,24 +56,32 @@ def _try_all(image, methods=None, figsize=None, num_cols=2, verbose=True):
     """
     from matplotlib import pyplot as plt
 
+    # Compute the image histogram for better performances
+    nbins = 256  # Default in threshold functions
+    hist = histogram(image.reshape(-1), nbins, source_range='image')
+
     # Handle default value
     methods = methods or {}
 
     num_rows = math.ceil((len(methods) + 1.) / num_cols)
     fig, ax = plt.subplots(num_rows, num_cols, figsize=figsize,
                            sharex=True, sharey=True)
-    ax = ax.ravel()
+    ax = ax.reshape(-1)
 
     ax[0].imshow(image, cmap=plt.cm.gray)
     ax[0].set_title('Original')
 
     i = 1
     for name, func in methods.items():
+        # Use precomputed histogram for supporting functions
+        sig = inspect.signature(func)
+        _kwargs = dict(hist=hist) if 'hist' in sig.parameters else {}
+
         ax[i].set_title(name)
         try:
-            ax[i].imshow(func(image), cmap=plt.cm.gray)
+            ax[i].imshow(func(image, **_kwargs), cmap=plt.cm.gray)
         except Exception as e:
-            ax[i].text(0.5, 0.5, "%s" % type(e).__name__,
+            ax[i].text(0.5, 0.5, f"{type(e).__name__}",
                        ha="center", va="center", transform=ax[i].transAxes)
         i += 1
         if verbose:
@@ -80,6 +94,7 @@ def _try_all(image, methods=None, figsize=None, num_cols=2, verbose=True):
     return fig, ax
 
 
+@require("matplotlib", ">=3.3")
 def try_all_threshold(image, figsize=(8, 5), verbose=True):
     """Returns a figure comparing the outputs of different thresholding methods.
 
@@ -139,7 +154,7 @@ def try_all_threshold(image, figsize=(8, 5), verbose=True):
                     methods=methods, verbose=verbose)
 
 
-def threshold_local(image, block_size, method='gaussian', offset=0,
+def threshold_local(image, block_size=3, method='gaussian', offset=0,
                     mode='reflect', param=None, cval=0):
     """Compute a threshold mask image based on local pixel neighborhood.
 
@@ -150,13 +165,13 @@ def threshold_local(image, block_size, method='gaussian', offset=0,
 
     Parameters
     ----------
-    image : (N, M) ndarray
-        Input image.
-    block_size : int
+    image : (N, M[, ..., P]) ndarray
+        Grayscale input image.
+    block_size : int or sequence of int
         Odd size of pixel neighborhood which is used to calculate the
         threshold value (e.g. 3, 5, 7, ..., 21, ...).
     method : {'generic', 'gaussian', 'mean', 'median'}, optional
-        Method used to determine adaptive threshold for local neighbourhood in
+        Method used to determine adaptive threshold for local neighborhood in
         weighted mean image.
 
         * 'generic': use custom function (see ``param`` parameter)
@@ -176,20 +191,22 @@ def threshold_local(image, block_size, method='gaussian', offset=0,
     param : {int, function}, optional
         Either specify sigma for 'gaussian' method or function object for
         'generic' method. This functions takes the flat array of local
-        neighbourhood as a single argument and returns the calculated
+        neighborhood as a single argument and returns the calculated
         threshold for the centre pixel.
     cval : float, optional
         Value to fill past edges of input if mode is 'constant'.
 
     Returns
     -------
-    threshold : (N, M) ndarray
+    threshold : (N, M[, ..., P]) ndarray
         Threshold image. All pixels in the input image higher than the
         corresponding pixel in the threshold image are considered foreground.
 
     References
     ----------
-    .. [1] https://docs.opencv.org/modules/imgproc/doc/miscellaneous_transformations.html?highlight=threshold#adaptivethreshold
+    .. [1] Gonzalez, R. C. and Wood, R. E. "Digital Image Processing
+           (2nd Edition)." Prentice-Hall Inc., 2002: 600--612.
+           ISBN: 0-201-18075-8
 
     Examples
     --------
@@ -199,30 +216,33 @@ def threshold_local(image, block_size, method='gaussian', offset=0,
     >>> func = lambda arr: arr.mean()
     >>> binary_image2 = image > threshold_local(image, 15, 'generic',
     ...                                         param=func)
+
     """
-    if block_size % 2 == 0:
-        raise ValueError("The kwarg ``block_size`` must be odd! Given "
-                         "``block_size`` {0} is even.".format(block_size))
-    check_nD(image, 2)
-    thresh_image = np.zeros(image.shape, 'double')
+
+    if np.isscalar(block_size):
+        block_size = (block_size,) * image.ndim
+    elif len(block_size) != image.ndim:
+        raise ValueError("len(block_size) must equal image.ndim.")
+    block_size = tuple(block_size)
+    if any(b % 2 == 0 for b in block_size):
+        raise ValueError(f'block_size must be odd! Given block_size '
+                         f'{block_size} contains even values.')
+    float_dtype = _supported_float_type(image.dtype)
+    image = image.astype(float_dtype, copy=False)
+    thresh_image = np.zeros(image.shape, dtype=float_dtype)
     if method == 'generic':
         ndi.generic_filter(image, param, block_size,
                            output=thresh_image, mode=mode, cval=cval)
     elif method == 'gaussian':
         if param is None:
             # automatically determine sigma which covers > 99% of distribution
-            sigma = (block_size - 1) / 6.0
+            sigma = tuple([(b - 1) / 6.0 for b in block_size])
         else:
             sigma = param
-        ndi.gaussian_filter(image, sigma, output=thresh_image, mode=mode,
-                            cval=cval)
+        gaussian(image, sigma, output=thresh_image, mode=mode, cval=cval)
     elif method == 'mean':
-        mask = 1. / block_size * np.ones((block_size,))
-        # separation of filters to speedup convolution
-        ndi.convolve1d(image, mask, axis=0, output=thresh_image, mode=mode,
-                       cval=cval)
-        ndi.convolve1d(thresh_image, mask, axis=1, output=thresh_image,
-                       mode=mode, cval=cval)
+        ndi.uniform_filter(image, block_size, output=thresh_image, mode=mode,
+                           cval=cval)
     elif method == 'median':
         ndi.median_filter(image, block_size, output=thresh_image, mode=mode,
                           cval=cval)
@@ -233,16 +253,80 @@ def threshold_local(image, block_size, method='gaussian', offset=0,
     return thresh_image - offset
 
 
-def threshold_otsu(image, nbins=256):
-    """Return threshold value based on Otsu's method.
+def _validate_image_histogram(image, hist, nbins=None, normalize=False):
+    """Ensure that either image or hist were given, return valid histogram.
+
+    If hist is given, image is ignored.
 
     Parameters
     ----------
-    image : (N, M) ndarray
+    image : array or None
+        Grayscale image.
+    hist : array, 2-tuple of array, or None
+        Histogram, either a 1D counts array, or an array of counts together
+        with an array of bin centers.
+    nbins : int, optional
+        The number of bins with which to compute the histogram, if `hist` is
+        None.
+    normalize : bool
+        If hist is not given, it will be computed by this function. This
+        parameter determines whether the computed histogram is normalized
+        (i.e. entries sum up to 1) or not.
+
+    Returns
+    -------
+    counts : 1D array of float
+        Each element is the number of pixels falling in each intensity bin.
+    bin_centers : 1D array
+        Each element is the value corresponding to the center of each intensity
+        bin.
+
+    Raises
+    ------
+    ValueError : if image and hist are both None
+    """
+    if image is None and hist is None:
+        raise Exception("Either image or hist must be provided.")
+
+    if hist is not None:
+        if isinstance(hist, (tuple, list)):
+            counts, bin_centers = hist
+        else:
+            counts = hist
+            bin_centers = np.arange(counts.size)
+
+        if counts[0] == 0 or counts[-1] == 0:
+            # Trim histogram from both ends by removing starting and
+            # ending zeroes as in histogram(..., source_range="image")
+            cond = counts > 0
+            start = np.argmax(cond)
+            end = cond.size - np.argmax(cond[::-1])
+            counts, bin_centers = counts[start:end], bin_centers[start:end]
+    else:
+        counts, bin_centers = histogram(
+            image.reshape(-1), nbins, source_range='image', normalize=normalize
+            )
+    return counts.astype('float32', copy=False), bin_centers
+
+
+def threshold_otsu(image=None, nbins=256, *, hist=None):
+    """Return threshold value based on Otsu's method.
+
+    Either image or hist must be provided. If hist is provided, the actual
+    histogram of the image is ignored.
+
+    Parameters
+    ----------
+    image : (N, M[, ..., P]) ndarray, optional
         Grayscale input image.
     nbins : int, optional
         Number of bins used to calculate histogram. This value is ignored for
         integer arrays.
+    hist : array, or 2-tuple of arrays, optional
+        Histogram from which to determine the threshold, and optionally a
+        corresponding array of bin center intensities. If no hist provided,
+        this function will compute it from the image.
+
 
     Returns
     -------
@@ -250,11 +334,6 @@ def threshold_otsu(image, nbins=256):
         Upper threshold value. All pixels with an intensity higher than
         this value are assumed to be foreground.
 
-    Raises
-    ------
-    ValueError
-         If ``image`` only contains a single grayscale value.
-
     References
     ----------
     .. [1] Wikipedia, https://en.wikipedia.org/wiki/Otsu's_Method
@@ -270,26 +349,26 @@ def threshold_otsu(image, nbins=256):
     -----
     The input image must be grayscale.
     """
-    if len(image.shape) > 2 and image.shape[-1] in (3, 4):
-        msg = "threshold_otsu is expected to work correctly only for " \
-              "grayscale images; image shape {0} looks like an RGB image"
-        warn(msg.format(image.shape))
+    if image is not None and image.ndim > 2 and image.shape[-1] in (3, 4):
+        warn(f'threshold_otsu is expected to work correctly only for '
+             f'grayscale images; image shape {image.shape} looks like '
+             f'that of an RGB image.')
 
-    # Check if the image is multi-colored or not
-    if image.min() == image.max():
-        raise ValueError("threshold_otsu is expected to work with images "
-                         "having more than one color. The input image seems "
-                         "to have just one color {0}.".format(image.min()))
+    # Check if the image has more than one intensity value; if not, return that
+    # value
+    if image is not None:
+        first_pixel = image.reshape(-1)[0]
+        if np.all(image == first_pixel):
+            return first_pixel
 
-    hist, bin_centers = histogram(image.ravel(), nbins, source_range='image')
-    hist = hist.astype(float)
+    counts, bin_centers = _validate_image_histogram(image, hist, nbins)
 
     # class probabilities for all possible thresholds
-    weight1 = np.cumsum(hist)
-    weight2 = np.cumsum(hist[::-1])[::-1]
+    weight1 = np.cumsum(counts)
+    weight2 = np.cumsum(counts[::-1])[::-1]
     # class means for all possible thresholds
-    mean1 = np.cumsum(hist * bin_centers) / weight1
-    mean2 = (np.cumsum((hist * bin_centers)[::-1]) / weight2[::-1])[::-1]
+    mean1 = np.cumsum(counts * bin_centers) / weight1
+    mean2 = (np.cumsum((counts * bin_centers)[::-1]) / weight2[::-1])[::-1]
 
     # Clip ends to align class 1 and class 2 variables:
     # The last value of ``weight1``/``mean1`` should pair with zero values in
@@ -297,20 +376,27 @@ def threshold_otsu(image, nbins=256):
     variance12 = weight1[:-1] * weight2[1:] * (mean1[:-1] - mean2[1:]) ** 2
 
     idx = np.argmax(variance12)
-    threshold = bin_centers[:-1][idx]
+    threshold = bin_centers[idx]
+
     return threshold
 
 
-def threshold_yen(image, nbins=256):
+def threshold_yen(image=None, nbins=256, *, hist=None):
     """Return threshold value based on Yen's method.
+    Either image or hist must be provided. In case hist is given, the actual
+    histogram of the image is ignored.
 
     Parameters
     ----------
-    image : (N, M) ndarray
-        Input image.
+    image : (N, M[, ..., P]) ndarray
+        Grayscale input image.
     nbins : int, optional
         Number of bins used to calculate histogram. This value is ignored for
         integer arrays.
+    hist : array, or 2-tuple of arrays, optional
+        Histogram from which to determine the threshold, and optionally a
+        corresponding array of bin center intensities.
+        An alternative use of this function is to pass it only hist.
 
     Returns
     -------
@@ -336,14 +422,15 @@ def threshold_yen(image, nbins=256):
     >>> thresh = threshold_yen(image)
     >>> binary = image <= thresh
     """
-    hist, bin_centers = histogram(image.ravel(), nbins, source_range='image')
+    counts, bin_centers = _validate_image_histogram(image, hist, nbins)
+
     # On blank images (e.g. filled with 0) with int dtype, `histogram()`
     # returns ``bin_centers`` containing only one value. Speed up with it.
     if bin_centers.size == 1:
         return bin_centers[0]
 
     # Calculate probability mass function
-    pmf = hist.astype(np.float32) / hist.sum()
+    pmf = counts.astype('float32', copy=False) / counts.sum()
     P1 = np.cumsum(pmf)  # Cumulative normalized histogram
     P1_sq = np.cumsum(pmf ** 2)
     # Get cumsum calculated from end of squared array:
@@ -355,7 +442,7 @@ def threshold_yen(image, nbins=256):
     return bin_centers[crit.argmax()]
 
 
-def threshold_isodata(image, nbins=256, return_all=False):
+def threshold_isodata(image=None, nbins=256, return_all=False, *, hist=None):
     """Return threshold value(s) based on ISODATA method.
 
     Histogram-based threshold, known as Ridler-Calvard method or inter-means.
@@ -371,16 +458,23 @@ def threshold_isodata(image, nbins=256, return_all=False):
     For integer images, the above equality holds to within one; for floating-
     point images, the equality holds to within the histogram bin-width.
 
+    Either image or hist must be provided. In case hist is given, the actual
+    histogram of the image is ignored.
+
     Parameters
     ----------
-    image : (N, M) ndarray
-        Input image.
+    image : (N, M[, ..., P]) ndarray
+        Grayscale input image.
     nbins : int, optional
         Number of bins used to calculate histogram. This value is ignored for
         integer arrays.
-    return_all: bool, optional
+    return_all : bool, optional
         If False (default), return only the lowest threshold that satisfies
         the above equality. If True, return all valid thresholds.
+    hist : array, or 2-tuple of arrays, optional
+        Histogram to determine the threshold from and a corresponding array
+        of bin center intensities. Alternatively, only the histogram can be
+        passed.
 
     Returns
     -------
@@ -408,7 +502,7 @@ def threshold_isodata(image, nbins=256, return_all=False):
     >>> thresh = threshold_isodata(image)
     >>> binary = image > thresh
     """
-    hist, bin_centers = histogram(image.ravel(), nbins, source_range='image')
+    counts, bin_centers = _validate_image_histogram(image, hist, nbins)
 
     # image only contains one unique value
     if len(bin_centers) == 1:
@@ -417,15 +511,15 @@ def threshold_isodata(image, nbins=256, return_all=False):
         else:
             return bin_centers[0]
 
-    hist = hist.astype(np.float32)
+    counts = counts.astype('float32', copy=False)
 
     # csuml and csumh contain the count of pixels in that bin or lower, and
     # in all bins strictly higher than that bin, respectively
-    csuml = np.cumsum(hist)
+    csuml = np.cumsum(counts)
     csumh = csuml[-1] - csuml
 
     # intensity_sum contains the total pixel intensity from each bin
-    intensity_sum = hist * bin_centers
+    intensity_sum = counts * bin_centers
 
     # l and h contain average value of all pixels in that bin or lower, and
     # in all bins strictly higher than that bin, respectively.
@@ -526,8 +620,8 @@ def threshold_li(image, *, tolerance=None, initial_guess=None,
 
     Parameters
     ----------
-    image : ndarray
-        Input image.
+    image : (N, M[, ..., P]) ndarray
+        Grayscale input image.
 
     tolerance : float, optional
         Finish the computation when the change in the threshold in an iteration
@@ -597,7 +691,10 @@ def threshold_li(image, *, tolerance=None, initial_guess=None,
     # Li's algorithm requires positive image (because of log(mean))
     image_min = np.min(image)
     image -= image_min
-    tolerance = tolerance or np.min(np.diff(np.unique(image))) / 2
+    if image.dtype.kind in 'iu':
+        tolerance = tolerance or 0.5
+    else:
+        tolerance = tolerance or np.min(np.diff(np.unique(image))) / 2
 
     # Initial estimate for iteration. See "initial_guess" in the parameter list
     if initial_guess is None:
@@ -608,9 +705,9 @@ def threshold_li(image, *, tolerance=None, initial_guess=None,
         t_next = initial_guess - image_min
         image_max = np.max(image) + image_min
         if not 0 < t_next < np.max(image):
-            msg = ('The initial guess for threshold_li must be within the '
-                   'range of the image. Got {} for image min {} and max {} '
-                   .format(initial_guess, image_min, image_max))
+            msg = (f'The initial guess for threshold_li must be within the '
+                   f'range of the image. Got {initial_guess} for image min '
+                   f'{image_min} and max {image_max}.')
             raise ValueError(msg)
     else:
         raise TypeError('Incorrect type for `initial_guess`; should be '
@@ -628,38 +725,75 @@ def threshold_li(image, *, tolerance=None, initial_guess=None,
 
     # Stop the iterations when the difference between the
     # new and old threshold values is less than the tolerance
-    while abs(t_next - t_curr) > tolerance:
-        t_curr = t_next
-        foreground = (image > t_curr)
-        mean_fore = np.mean(image[foreground])
-        mean_back = np.mean(image[~foreground])
+    # or if the background mode has only one value left,
+    # since log(0) is not defined.
 
-        t_next = ((mean_back - mean_fore) /
-                  (np.log(mean_back) - np.log(mean_fore)))
+    if image.dtype.kind in 'iu':
+        hist, bin_centers = histogram(image.reshape(-1),
+                                      source_range='image')
+        hist = hist.astype('float32', copy=False)
+        while abs(t_next - t_curr) > tolerance:
+            t_curr = t_next
+            foreground = bin_centers > t_curr
+            background = ~foreground
 
-        if iter_callback is not None:
-            iter_callback(t_next + image_min)
+            mean_fore = np.average(bin_centers[foreground],
+                                   weights=hist[foreground])
+            mean_back = np.average(bin_centers[background],
+                                   weights=hist[background])
+
+            if mean_back == 0:
+                break
+
+            t_next = ((mean_back - mean_fore)
+                      / (np.log(mean_back) - np.log(mean_fore)))
+
+            if iter_callback is not None:
+                iter_callback(t_next + image_min)
+
+    else:
+        while abs(t_next - t_curr) > tolerance:
+            t_curr = t_next
+            foreground = (image > t_curr)
+            mean_fore = np.mean(image[foreground])
+            mean_back = np.mean(image[~foreground])
+
+            if mean_back == 0.0:
+                break
+
+            t_next = ((mean_back - mean_fore)
+                      / (np.log(mean_back) - np.log(mean_fore)))
+
+            if iter_callback is not None:
+                iter_callback(t_next + image_min)
 
     threshold = t_next + image_min
     return threshold
 
 
-def threshold_minimum(image, nbins=256, max_iter=10000):
+def threshold_minimum(image=None, nbins=256, max_num_iter=10000, *, hist=None):
     """Return threshold value based on minimum method.
 
-    The histogram of the input ``image`` is computed and smoothed until
-    there are only two maxima. Then the minimum in between is the threshold
-    value.
+    The histogram of the input ``image`` is computed if not provided and
+    smoothed until there are only two maxima. Then the minimum in between is
+    the threshold value.
+
+    Either image or hist must be provided. In case hist is given, the actual
+    histogram of the image is ignored.
 
     Parameters
     ----------
-    image : (M, N) ndarray
-        Input image.
+    image : (N, M[, ..., P]) ndarray, optional
+        Grayscale input image.
     nbins : int, optional
         Number of bins used to calculate histogram. This value is ignored for
         integer arrays.
-    max_iter: int, optional
+    max_num_iter : int, optional
         Maximum number of iterations to smooth the histogram.
+    hist : array, or 2-tuple of arrays, optional
+        Histogram to determine the threshold from and a corresponding array
+        of bin center intensities. Alternatively, only the histogram can be
+        passed.
 
     Returns
     -------
@@ -707,11 +841,11 @@ def threshold_minimum(image, nbins=256, max_iter=10000):
 
         return maximum_idxs
 
-    hist, bin_centers = histogram(image.ravel(), nbins, source_range='image')
+    counts, bin_centers = _validate_image_histogram(image, hist, nbins)
 
-    smooth_hist = hist.astype(np.float64, copy=False)
+    smooth_hist = counts.astype('float32', copy=False)
 
-    for counter in range(max_iter):
+    for counter in range(max_num_iter):
         smooth_hist = ndi.uniform_filter1d(smooth_hist, 3)
         maximum_idxs = find_local_maxima_idx(smooth_hist)
         if len(maximum_idxs) < 3:
@@ -719,7 +853,7 @@ def threshold_minimum(image, nbins=256, max_iter=10000):
 
     if len(maximum_idxs) != 2:
         raise RuntimeError('Unable to find two maxima in histogram')
-    elif counter == max_iter - 1:
+    elif counter == max_num_iter - 1:
         raise RuntimeError('Maximum iteration reached for histogram'
                            'smoothing')
 
@@ -795,13 +929,14 @@ def threshold_triangle(image, nbins=256):
     """
     # nbins is ignored for integer arrays
     # so, we recalculate the effective nbins.
-    hist, bin_centers = histogram(image.ravel(), nbins, source_range='image')
+    hist, bin_centers = histogram(image.reshape(-1), nbins,
+                                  source_range='image')
     nbins = len(hist)
 
     # Find peak, lowest and highest gray levels.
     arg_peak_height = np.argmax(hist)
     peak_height = hist[arg_peak_height]
-    arg_low_level, arg_high_level = np.where(hist > 0)[0][[0, -1]]
+    arg_low_level, arg_high_level = np.flatnonzero(hist)[[0, -1]]
 
     # Flip is True if left tail is shorter.
     flip = arg_peak_height - arg_low_level < arg_high_level - arg_peak_height
@@ -837,26 +972,6 @@ def threshold_triangle(image, nbins=256):
     return bin_centers[arg_level]
 
 
-def _validate_window_size(axis_sizes):
-    """Ensure all sizes in ``axis_sizes`` are odd.
-
-    Parameters
-    ----------
-    axis_sizes : iterable of int
-
-    Raises
-    ------
-    ValueError
-        If any given axis size is even.
-    """
-    for axis_size in axis_sizes:
-        if axis_size % 2 == 0:
-            msg = ('Window size for `threshold_sauvola` or '
-                   '`threshold_niblack` must not be even on any dimension. '
-                   'Got {}'.format(axis_sizes))
-            raise ValueError(msg)
-
-
 def _mean_std(image, w):
     """Return local mean and standard deviation of each pixel using a
     neighborhood defined by a rectangular window size ``w``.
@@ -865,8 +980,8 @@ def _mean_std(image, w):
 
     Parameters
     ----------
-    image : ndarray
-        Input image.
+    image : (N, M[, ..., P]) ndarray
+        Grayscale input image.
     w : int, or iterable of int
         Window size specified as a single odd integer (3, 5, 7, …),
         or an iterable of length ``image.ndim`` containing only odd
@@ -887,27 +1002,37 @@ def _mean_std(image, w):
            Retrieval XV, (San Jose, USA), Jan. 2008.
            :DOI:`10.1117/12.767755`
     """
+
     if not isinstance(w, Iterable):
         w = (w,) * image.ndim
     _validate_window_size(w)
 
+    float_dtype = _supported_float_type(image.dtype)
     pad_width = tuple((k // 2 + 1, k // 2) for k in w)
-    padded = np.pad(image.astype('float'), pad_width,
+    padded = np.pad(image.astype(float_dtype, copy=False), pad_width,
                     mode='reflect')
-    padded_sq = padded * padded
 
-    integral = integral_image(padded)
-    integral_sq = integral_image(padded_sq)
+    # Note: keep float64 integral images for accuracy. Outputs of
+    # _correlate_sparse can later be safely cast to float_dtype
+    integral = integral_image(padded, dtype=np.float64)
+    padded *= padded
+    integral_sq = integral_image(padded, dtype=np.float64)
 
-    kern = np.zeros(tuple(k + 1 for k in w))
-    for indices in itertools.product(*([[0, -1]] * image.ndim)):
-        kern[indices] = (-1) ** (image.ndim % 2 != np.sum(indices) % 2)
+    # Create lists of non-zero kernel indices and values
+    kernel_indices = list(itertools.product(*tuple([(0, _w) for _w in w])))
+    kernel_values = [(-1) ** (image.ndim % 2 != np.sum(indices) % 2)
+                     for indices in kernel_indices]
 
-    total_window_size = np.prod(w)
-    sum_full = ndi.correlate(integral, kern, mode='constant')
-    m = crop(sum_full, pad_width) / total_window_size
-    sum_sq_full = ndi.correlate(integral_sq, kern, mode='constant')
-    g2 = crop(sum_sq_full, pad_width) / total_window_size
+    total_window_size = math.prod(w)
+    kernel_shape = tuple(_w + 1 for _w in w)
+    m = _correlate_sparse(integral, kernel_shape, kernel_indices,
+                          kernel_values)
+    m = m.astype(float_dtype, copy=False)
+    m /= total_window_size
+    g2 = _correlate_sparse(integral_sq, kernel_shape, kernel_indices,
+                           kernel_values)
+    g2 = g2.astype(float_dtype, copy=False)
+    g2 /= total_window_size
     # Note: we use np.clip because g2 is not guaranteed to be greater than
     # m*m when floating point error is considered
     s = np.sqrt(np.clip(g2 - m * m, 0, None))
@@ -929,8 +1054,8 @@ def threshold_niblack(image, window_size=15, k=0.2):
 
     Parameters
     ----------
-    image: ndarray
-        Input image.
+    image : (N, M[, ..., P]) ndarray
+        Grayscale input image.
     window_size : int, or iterable of int, optional
         Window size specified as a single odd integer (3, 5, 7, …),
         or an iterable of length ``image.ndim`` containing only odd
@@ -990,12 +1115,12 @@ def threshold_sauvola(image, window_size=15, k=0.2, r=None):
     pixel (x,y) neighborhood defined by a rectangular window with size w
     times w centered around the pixel. k is a configurable parameter
     that weights the effect of standard deviation.
-    R is the maximum standard deviation of a greyscale image.
+    R is the maximum standard deviation of a grayscale image.
 
     Parameters
     ----------
-    image: ndarray
-        Input image.
+    image : (N, M[, ..., P]) ndarray
+        Grayscale input image.
     window_size : int, or iterable of int, optional
         Window size specified as a single odd integer (3, 5, 7, …),
         or an iterable of length ``image.ndim`` containing only odd
@@ -1084,16 +1209,20 @@ def apply_hysteresis_threshold(image, low, high):
     return thresholded
 
 
-def threshold_multiotsu(image, classes=3, nbins=256):
-    r"""Generate `classes`-1 threshold values to divide gray levels in `image`.
+def threshold_multiotsu(image=None, classes=3, nbins=256, *, hist=None):
+    r"""Generate `classes`-1 threshold values to divide gray levels in `image`,
+    following Otsu's method for multiple classes.
 
     The threshold values are chosen to maximize the total sum of pairwise
     variances between the thresholded graylevel classes. See Notes and [1]_
     for more details.
 
+    Either image or hist must be provided. If hist is provided, the actual
+    histogram of the image is ignored.
+
     Parameters
     ----------
-    image : (N, M) ndarray
+    image : (N, M[, ..., P]) ndarray, optional
         Grayscale input image.
     classes : int, optional
         Number of classes to be thresholded, i.e. the number of resulting
@@ -1101,12 +1230,22 @@ def threshold_multiotsu(image, classes=3, nbins=256):
     nbins : int, optional
         Number of bins used to calculate the histogram. This value is ignored
         for integer arrays.
+    hist : array, or 2-tuple of arrays, optional
+        Histogram from which to determine the threshold, and optionally a
+        corresponding array of bin center intensities. If no hist provided,
+        this function will compute it from the image (see notes).
 
     Returns
     -------
-    idx_thresh : array
+    thresh : array
         Array containing the threshold values for the desired classes.
 
+    Raises
+    ------
+    ValueError
+         If ``image`` contains less grayscale value then the desired
+         number of classes.
+
     Notes
     -----
     This implementation relies on a Cython function whose complexity
@@ -1114,12 +1253,20 @@ def threshold_multiotsu(image, classes=3, nbins=256):
     is the number of histogram bins and :math:`C` is the number of
     classes desired.
 
+    If no hist is given, this function will make use of
+    `skimage.exposure.histogram`, which behaves differently than
+    `np.histogram`. While both allowed, use the former for consistent
+    behaviour.
+
+    The input image must be grayscale.
+
     References
     ----------
     .. [1] Liao, P-S., Chen, T-S. and Chung, P-C., "A fast algorithm for
            multilevel thresholding", Journal of Information Science and
            Engineering 17 (5): 713-727, 2001. Available at:
-           <http://ftp.iis.sinica.edu.tw/JISE/2001/200109_01.pdf>
+           <https://ftp.iis.sinica.edu.tw/JISE/2001/200109_01.pdf>
+           :DOI:`10.6688/JISE.2001.17.5.1`
     .. [2] Tosa, Y., "Multi-Otsu Threshold", a java plugin for ImageJ.
            Available at:
            <http://imagej.net/plugins/download/Multi_OtsuThreshold.java>
@@ -1133,48 +1280,36 @@ def threshold_multiotsu(image, classes=3, nbins=256):
     >>> regions = np.digitize(image, bins=thresholds)
     >>> regions_colorized = label2rgb(regions)
     """
+    if image is not None and image.ndim > 2 and image.shape[-1] in (3, 4):
+        warn(f'threshold_multiotsu is expected to work correctly only for '
+             f'grayscale images; image shape {image.shape} looks like '
+             f'that of an RGB image.')
+
     # calculating the histogram and the probability of each gray level.
-    hist, bin_centers = histogram(image.ravel(),
-                                  nbins=nbins,
-                                  source_range='image')
-    prob = hist / image.size
-    # histogram ignores nbins for integer arrays.
-    nbins = len(bin_centers)
+    prob, bin_centers = _validate_image_histogram(image, hist, nbins,
+                                                  normalize=True)
+    prob = prob.astype('float32', copy=False)
 
-    # Compute the zeroth (momP, cumulative probability) and first
-    # (momS, mean) moments
+    nvalues = np.count_nonzero(prob)
+    if nvalues < classes:
+        msg = (f'After discretization into bins, the input image has '
+               f'only {nvalues} different values. It cannot be thresholded '
+               f'in {classes} classes. If there are more unique values '
+               f'before discretization, try increasing the number of bins '
+               f'(`nbins`).')
+        raise ValueError(msg)
+    elif nvalues == classes:
+        thresh_idx = np.flatnonzero(prob)[:-1]
+    else:
+        # Get threshold indices
+        try:
+            thresh_idx = _get_multiotsu_thresh_indices_lut(prob, classes - 1)
+        except MemoryError:
+            # Don't use LUT if the number of bins is too large (if the
+            # image is uint16 for example): in this case, the
+            # allocated memory is too large.
+            thresh_idx = _get_multiotsu_thresh_indices(prob, classes - 1)
 
-    # building the lookup tables.
-    # step 1: calculating the diagonal.
-    prob[0] = 0
-    momP = np.diagflat(prob)
-    momS = np.diagflat(np.arange(nbins) * prob)
+    thresh = bin_centers[thresh_idx]
 
-    # step 2: calculating the first row.
-    momP[1, 2:] = prob[1] + np.cumsum(prob[2:])
-    momS[1, 2:] = prob[1] + np.cumsum(np.arange(2, nbins) * prob[2:])
-
-    # step 3: the other rows are recursively computed as:
-    # A[i, j] = A[1, j] - A[1, i-1] for A in {momP, momS};  i > 1 and j > i.
-    upper_tri = np.triu_indices_from(momP[2:, 2:], 1)
-
-    momP[2:, 2:][upper_tri] = (momP[1, 2:][upper_tri[1]]
-                               - np.repeat(momP[1, 1:-2],
-                                           np.arange(nbins - 3, 0, -1)))
-    momS[2:, 2:][upper_tri] = (momS[1, 2:][upper_tri[1]]
-                               - np.repeat(momS[1, 1:-2],
-                                           np.arange(nbins - 3, 0, -1)))
-
-    # step 4: calculating the between class variance.
-    var_btwcls = np.zeros_like(momP)
-    idx = momP > 0
-    var_btwcls[idx] = momS[idx] ** 2 / momP[idx]
-
-    # finding max threshold candidates, depending on classes.
-    # number of thresholds is equal to number of classes - 1.
-    aux_thresh = _find_threshold_multiotsu(var_btwcls, classes, nbins)
-
-    # correcting threshold values.
-    idx_thresh = bin_centers[:-1][aux_thresh.astype('int')]
-
-    return idx_thresh
+    return thresh
