@@ -1,11 +1,9 @@
 from ._find_contours import find_contours
-from ._marching_cubes_lewiner import marching_cubes_lewiner, marching_cubes
-from ._marching_cubes_classic import (marching_cubes_classic,
-                                      mesh_surface_area)
+from ._marching_cubes_lewiner import marching_cubes, mesh_surface_area
 from ._regionprops import (regionprops, perimeter,
                            perimeter_crofton, euler_number, regionprops_table)
 from ._polygon import approximate_polygon, subdivide_polygon
-from .pnpoly import points_in_poly, grid_points_in_poly
+from .pnpoly import (points_in_poly, grid_points_in_poly)
 from ._moments import (moments, moments_central, moments_coords,
                        moments_coords_central, moments_normalized, centroid,
                        moments_hu, inertia_tensor, inertia_tensor_eigvals)
@@ -14,7 +12,9 @@ from .fit import LineModelND, CircleModel, EllipseModel, ransac
 from .block import block_reduce
 from ._label import label
 from .entropy import shannon_entropy
-
+from ._blur_effect import blur_effect
+from ._colocalization import (pearson_corr_coeff, manders_coloc_coeff,
+                              manders_overlap_coeff, intersection_coeff)
 
 __all__ = ['find_contours',
            'regionprops',
@@ -38,12 +38,15 @@ __all__ = ['find_contours',
            'inertia_tensor',
            'inertia_tensor_eigvals',
            'marching_cubes',
-           'marching_cubes_lewiner',
-           'marching_cubes_classic',
            'mesh_surface_area',
            'profile_line',
            'label',
            'points_in_poly',
            'grid_points_in_poly',
            'shannon_entropy',
+           'blur_effect',
+           'pearson_corr_coeff',
+           'manders_coloc_coeff',
+           'manders_overlap_coeff',
+           'intersection_coeff',
            ]
