@@ -5,18 +5,18 @@ import pytest
 from numpy.testing import assert_equal
 
 from skimage import data, filters, img_as_float
-from skimage._shared.testing import test_parallel, expected_warnings
+from skimage._shared.testing import run_in_parallel, expected_warnings
 from skimage.segmentation import slic
 
 
-@test_parallel()
+@run_in_parallel()
 def test_color_2d():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     img = np.zeros((20, 21, 3))
     img[:10, :10, 0] = 1
     img[10:, :10, 1] = 1
     img[10:, 10:, 2] = 1
-    img += 0.01 * rnd.normal(size=img.shape)
+    img += 0.01 * rng.normal(size=img.shape)
     img[img > 1] = 1
     img[img < 0] = 0
     seg = slic(img, n_segments=4, sigma=0, enforce_connectivity=False,
@@ -31,20 +31,14 @@ def test_color_2d():
     assert_equal(seg[10:, 10:], 3)
 
 
-def test_max_iter_kwarg_deprecation():
-    img = np.zeros((20, 21, 3))
-    with expected_warnings(["`max_iter` is a deprecated argument"]):
-        slic(img, max_iter=10, start_label=0)
-
-
 def test_multichannel_2d():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     img = np.zeros((20, 20, 8))
     img[:10, :10, 0:2] = 1
     img[:10, 10:, 2:4] = 1
     img[10:, :10, 4:6] = 1
     img[10:, 10:, 6:8] = 1
-    img += 0.01 * rnd.normal(size=img.shape)
+    img += 0.01 * rng.normal(size=img.shape)
     img = np.clip(img, 0, 1, out=img)
     seg = slic(img, n_segments=4, enforce_connectivity=False, start_label=0)
 
@@ -58,12 +52,12 @@ def test_multichannel_2d():
 
 
 def test_gray_2d():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     img = np.zeros((20, 21))
     img[:10, :10] = 0.33
     img[10:, :10] = 0.67
     img[10:, 10:] = 1.00
-    img += 0.0033 * rnd.normal(size=img.shape)
+    img += 0.0033 * rng.normal(size=img.shape)
     img[img > 1] = 1
     img[img < 0] = 0
     seg = slic(img, sigma=0, n_segments=4, compactness=1,
@@ -77,29 +71,14 @@ def test_gray_2d():
     assert_equal(seg[10:, 10:], 3)
 
 
-def test_gray_2d_deprecated_multichannel():
-    rnd = np.random.default_rng(0)
+def test_gray2d_default_channel_axis():
     img = np.zeros((20, 21))
     img[:10, :10] = 0.33
-    img[10:, :10] = 0.67
-    img[10:, 10:] = 1.00
-    img += 0.0033 * rnd.normal(size=img.shape)
-    img[img > 1] = 1
-    img[img < 0] = 0
-    with expected_warnings(["`multichannel` is a deprecated argument"]):
-        seg = slic(img, sigma=0, n_segments=4, compactness=1,
-                   multichannel=False, convert2lab=False, start_label=0)
-
-    assert_equal(len(np.unique(seg)), 4)
-    assert_equal(seg.shape, img.shape)
-    assert_equal(seg[:10, :10], 0)
-    assert_equal(seg[10:, :10], 2)
-    assert_equal(seg[:10, 10:], 1)
-    assert_equal(seg[10:, 10:], 3)
-
-    with expected_warnings(["Providing the `multichannel` argument"]):
-        seg = slic(img, 4, 1, 10, 0, None, False, convert2lab=False,
-                   start_label=0)
+    with pytest.raises(
+            ValueError, match="channel_axis=-1 indicates multichannel"
+    ):
+        slic(img)
+    slic(img, channel_axis=None)
 
 
 def _check_segment_labels(seg1, seg2, allowed_mismatch_ratio=0.1):
@@ -115,14 +94,18 @@ def test_slic_consistency_across_image_magnitude():
     img_uint16 = 256 * img_uint8.astype(np.uint16)
     img_float32 = img_as_float(img_uint8)
     img_float32_norm = img_float32 / img_float32.max()
+    img_float32_offset = img_float32 + 1000
 
     seg1 = slic(img_uint8)
     seg2 = slic(img_uint16)
     seg3 = slic(img_float32)
     seg4 = slic(img_float32_norm)
+    seg5 = slic(img_float32_offset)
 
     np.testing.assert_array_equal(seg1, seg2)
     np.testing.assert_array_equal(seg1, seg3)
+    # Assert that offset has no impact on result
+    np.testing.assert_array_equal(seg4, seg5)
     # Floating point cases can have mismatch due to floating point error
     # exact match was observed on x86_64, but mismatches seen no i686.
     # For now just verify that a similar number of superpixels are present in
@@ -133,7 +116,7 @@ def test_slic_consistency_across_image_magnitude():
 
 
 def test_color_3d():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     img = np.zeros((20, 21, 22, 3))
     slices = []
     for dim_size in img.shape[:-1]:
@@ -143,7 +126,7 @@ def test_color_3d():
     colors = list(product(*(([0, 1],) * 3)))
     for s, c in zip(slices, colors):
         img[s] = c
-    img += 0.01 * rnd.normal(size=img.shape)
+    img += 0.01 * rng.normal(size=img.shape)
     img[img > 1] = 1
     img[img < 0] = 0
     seg = slic(img, sigma=0, n_segments=8, start_label=0)
@@ -154,7 +137,7 @@ def test_color_3d():
 
 
 def test_gray_3d():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     img = np.zeros((20, 21, 22))
     slices = []
     for dim_size in img.shape:
@@ -164,7 +147,7 @@ def test_gray_3d():
     shades = np.arange(0, 1.000001, 1.0 / 7)
     for s, sh in zip(slices, shades):
         img[s] = sh
-    img += 0.001 * rnd.normal(size=img.shape)
+    img += 0.001 * rng.normal(size=img.shape)
     img[img > 1] = 1
     img[img < 0] = 0
     seg = slic(img, sigma=0, n_segments=8, compactness=1,
@@ -176,10 +159,10 @@ def test_gray_3d():
 
 
 def test_list_sigma():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     img = np.array([[1, 1, 1, 0, 0, 0],
                     [0, 0, 0, 1, 1, 1]], float)
-    img += 0.1 * rnd.normal(size=img.shape)
+    img += 0.1 * rng.normal(size=img.shape)
     result_sigma = np.array([[0, 0, 0, 1, 1, 1],
                              [0, 0, 0, 1, 1, 1]], int)
     with expected_warnings(["Input image is 2D: sigma number of "
@@ -190,14 +173,14 @@ def test_list_sigma():
 
 
 def test_spacing():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     img = np.array([[1, 1, 1, 0, 0],
                     [1, 1, 0, 0, 0]], float)
     result_non_spaced = np.array([[0, 0, 0, 1, 1],
                                   [0, 0, 1, 1, 1]], int)
     result_spaced = np.array([[0, 0, 0, 0, 0],
                               [1, 1, 1, 1, 1]], int)
-    img += 0.1 * rnd.normal(size=img.shape)
+    img += 0.1 * rng.normal(size=img.shape)
     seg_non_spaced = slic(img, n_segments=2, sigma=0, channel_axis=None,
                           compactness=1.0, start_label=0)
     seg_spaced = slic(img, n_segments=2, sigma=0, spacing=[500, 1],
@@ -220,10 +203,12 @@ def test_enforce_connectivity():
 
     segments_connected = slic(img, 2, compactness=0.0001,
                               enforce_connectivity=True,
-                              convert2lab=False, start_label=0)
+                              convert2lab=False, start_label=0,
+                              channel_axis=None)
     segments_disconnected = slic(img, 2, compactness=0.0001,
                                  enforce_connectivity=False,
-                                 convert2lab=False, start_label=0)
+                                 convert2lab=False, start_label=0,
+                                 channel_axis=None)
 
     # Make sure nothing fatal occurs (e.g. buffer overflow) at low values of
     # max_size_factor
@@ -231,7 +216,8 @@ def test_enforce_connectivity():
                                       enforce_connectivity=True,
                                       convert2lab=False,
                                       max_size_factor=0.8,
-                                      start_label=0)
+                                      start_label=0,
+                                      channel_axis=None)
 
     result_connected = np.array([[0, 0, 0, 1, 1, 1],
                                  [0, 0, 0, 1, 1, 1],
@@ -248,12 +234,12 @@ def test_enforce_connectivity():
 
 def test_slic_zero():
     # Same as test_color_2d but with slic_zero=True
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     img = np.zeros((20, 21, 3))
     img[:10, :10, 0] = 1
     img[10:, :10, 1] = 1
     img[10:, 10:, 2] = 1
-    img += 0.01 * rnd.normal(size=img.shape)
+    img += 0.01 * rng.normal(size=img.shape)
     img[img > 1] = 1
     img[img < 0] = 0
     seg = slic(img, n_segments=4, sigma=0, slic_zero=True, start_label=0)
@@ -268,12 +254,12 @@ def test_slic_zero():
 
 
 def test_more_segments_than_pixels():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     img = np.zeros((20, 21))
     img[:10, :10] = 0.33
     img[10:, :10] = 0.67
     img[10:, 10:] = 1.00
-    img += 0.0033 * rnd.normal(size=img.shape)
+    img += 0.0033 * rng.normal(size=img.shape)
     img[img > 1] = 1
     img[img < 0] = 0
     seg = slic(img, sigma=0, n_segments=500, compactness=1,
@@ -282,14 +268,14 @@ def test_more_segments_than_pixels():
 
 
 def test_color_2d_mask():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     msk = np.zeros((20, 21))
     msk[2:-2, 2:-2] = 1
     img = np.zeros((20, 21, 3))
     img[:10, :10, 0] = 1
     img[10:, :10, 1] = 1
     img[10:, 10:, 2] = 1
-    img += 0.01 * rnd.normal(size=img.shape)
+    img += 0.01 * rng.normal(size=img.shape)
     np.clip(img, 0, 1, out=img)
     seg = slic(img, n_segments=4, sigma=0, enforce_connectivity=False,
                mask=msk)
@@ -310,7 +296,7 @@ def test_color_2d_mask():
 
 
 def test_multichannel_2d_mask():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     msk = np.zeros((20, 20))
     msk[2:-2, 2:-2] = 1
     img = np.zeros((20, 20, 8))
@@ -318,7 +304,7 @@ def test_multichannel_2d_mask():
     img[:10, 10:, 2:4] = 1
     img[10:, :10, 4:6] = 1
     img[10:, 10:, 6:8] = 1
-    img += 0.01 * rnd.normal(size=img.shape)
+    img += 0.01 * rng.normal(size=img.shape)
     np.clip(img, 0, 1, out=img)
     seg = slic(img, n_segments=4, enforce_connectivity=False,
                mask=msk)
@@ -339,14 +325,14 @@ def test_multichannel_2d_mask():
 
 
 def test_gray_2d_mask():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     msk = np.zeros((20, 21))
     msk[2:-2, 2:-2] = 1
     img = np.zeros((20, 21))
     img[:10, :10] = 0.33
     img[10:, :10] = 0.67
     img[10:, 10:] = 1.00
-    img += 0.0033 * rnd.normal(size=img.shape)
+    img += 0.0033 * rng.normal(size=img.shape)
     np.clip(img, 0, 1, out=img)
     seg = slic(img, sigma=0, n_segments=4, compactness=1,
                channel_axis=None, convert2lab=False, mask=msk)
@@ -366,12 +352,12 @@ def test_gray_2d_mask():
 
 
 def test_list_sigma_mask():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     msk = np.zeros((2, 6))
     msk[:, 1:-1] = 1
     img = np.array([[1, 1, 1, 0, 0, 0],
                     [0, 0, 0, 1, 1, 1]], float)
-    img += 0.1 * rnd.normal(size=img.shape)
+    img += 0.1 * rng.normal(size=img.shape)
     result_sigma = np.array([[0, 1, 1, 2, 2, 0],
                              [0, 1, 1, 2, 2, 0]], int)
     seg_sigma = slic(img, n_segments=2, sigma=[50, 1],
@@ -380,7 +366,7 @@ def test_list_sigma_mask():
 
 
 def test_spacing_mask():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     msk = np.zeros((2, 5))
     msk[:, 1:-1] = 1
     img = np.array([[1, 1, 1, 0, 0],
@@ -389,7 +375,7 @@ def test_spacing_mask():
                                   [0, 1, 2, 2, 0]], int)
     result_spaced = np.array([[0, 1, 1, 1, 0],
                               [0, 2, 2, 2, 0]], int)
-    img += 0.1 * rnd.normal(size=img.shape)
+    img += 0.1 * rng.normal(size=img.shape)
     seg_non_spaced = slic(img, n_segments=2, sigma=0, channel_axis=None,
                           compactness=1.0, mask=msk)
     seg_spaced = slic(img, n_segments=2, sigma=0, spacing=[50, 1],
@@ -407,17 +393,18 @@ def test_enforce_connectivity_mask():
 
     segments_connected = slic(img, 2, compactness=0.0001,
                               enforce_connectivity=True,
-                              convert2lab=False, mask=msk)
+                              convert2lab=False, mask=msk, channel_axis=None)
     segments_disconnected = slic(img, 2, compactness=0.0001,
                                  enforce_connectivity=False,
-                                 convert2lab=False, mask=msk)
+                                 convert2lab=False, mask=msk, channel_axis=None)
 
     # Make sure nothing fatal occurs (e.g. buffer overflow) at low values of
     # max_size_factor
     segments_connected_low_max = slic(img, 2, compactness=0.0001,
                                       enforce_connectivity=True,
                                       convert2lab=False,
-                                      max_size_factor=0.8, mask=msk)
+                                      max_size_factor=0.8, mask=msk,
+                                      channel_axis=None)
 
     result_connected = np.array([[0, 1, 1, 2, 2, 0],
                                  [0, 1, 1, 2, 2, 0],
@@ -434,14 +421,14 @@ def test_enforce_connectivity_mask():
 
 def test_slic_zero_mask():
 
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     msk = np.zeros((20, 21))
     msk[2:-2, 2:-2] = 1
     img = np.zeros((20, 21, 3))
     img[:10, :10, 0] = 1
     img[10:, :10, 1] = 1
     img[10:, 10:, 2] = 1
-    img += 0.01 * rnd.normal(size=img.shape)
+    img += 0.01 * rng.normal(size=img.shape)
     np.clip(img, 0, 1, out=img)
     seg = slic(img, n_segments=4, sigma=0, slic_zero=True,
                mask=msk)
@@ -462,14 +449,14 @@ def test_slic_zero_mask():
 
 
 def test_more_segments_than_pixels_mask():
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     msk = np.zeros((20, 21))
     msk[2:-2, 2:-2] = 1
     img = np.zeros((20, 21))
     img[:10, :10] = 0.33
     img[10:, :10] = 0.67
     img[10:, 10:] = 1.00
-    img += 0.0033 * rnd.normal(size=img.shape)
+    img += 0.0033 * rng.normal(size=img.shape)
     np.clip(img, 0, 1, out=img)
     seg = slic(img, sigma=0, n_segments=500, compactness=1,
                channel_axis=None, convert2lab=False, mask=msk)
@@ -483,7 +470,7 @@ def test_color_3d_mask():
     msk = np.zeros((20, 21, 22))
     msk[2:-2, 2:-2, 2:-2] = 1
 
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     img = np.zeros((20, 21, 22, 3))
     slices = []
     for dim_size in msk.shape:
@@ -493,7 +480,7 @@ def test_color_3d_mask():
     colors = list(product(*(([0, 1],) * 3)))
     for s, c in zip(slices, colors):
         img[s] = c
-    img += 0.01 * rnd.normal(size=img.shape)
+    img += 0.01 * rng.normal(size=img.shape)
     np.clip(img, 0, 1, out=img)
 
     seg = slic(img, sigma=0, n_segments=8, mask=msk)
@@ -509,7 +496,7 @@ def test_gray_3d_mask():
     msk = np.zeros((20, 21, 22))
     msk[2:-2, 2:-2, 2:-2] = 1
 
-    rnd = np.random.default_rng(0)
+    rng = np.random.default_rng(0)
     img = np.zeros((20, 21, 22))
     slices = []
     for dim_size in img.shape:
@@ -519,7 +506,7 @@ def test_gray_3d_mask():
     shades = np.linspace(0, 1, 8)
     for s, sh in zip(slices, shades):
         img[s] = sh
-    img += 0.001 * rnd.normal(size=img.shape)
+    img += 0.001 * rng.normal(size=img.shape)
     np.clip(img, 0, 1, out=img)
     seg = slic(img, sigma=0, n_segments=8, channel_axis=None,
                convert2lab=False, mask=msk)
@@ -537,7 +524,7 @@ def test_dtype_support(dtype):
     img = np.random.rand(28, 28).astype(dtype)
 
     # Simply run the function to assert that it runs without error
-    slic(img, start_label=1)
+    slic(img, start_label=1, channel_axis=None)
 
 
 def test_start_label_fix():
@@ -558,3 +545,24 @@ def test_start_label_fix():
                   n_segments=6, compactness=0.01, enforce_connectivity=True,
                   max_num_iter=10)
     assert superp.min() == start_label
+
+
+def test_raises_ValueError_if_input_has_NaN():
+    img = np.zeros((4,5), dtype=float)
+    img[2, 3] = np.NaN
+    with pytest.raises(ValueError):
+        slic(img, channel_axis=None)
+
+    mask = ~np.isnan(img)
+    slic(img, mask=mask, channel_axis=None)
+
+
+@pytest.mark.parametrize("inf", [-np.inf, np.inf])
+def test_raises_ValueError_if_input_has_inf(inf):
+    img = np.zeros((4,5), dtype=float)
+    img[2, 3] = inf
+    with pytest.raises(ValueError):
+        slic(img, channel_axis=None)
+
+    mask = np.isfinite(img)
+    slic(img, mask=mask, channel_axis=None)
