@@ -32,42 +32,45 @@ https://github.com/scikit-image/scikit-image/issues/3405
 import os
 import argparse
 from datetime import datetime
-from collections import OrderedDict
-import string
 from warnings import warn
 
 from github import Github
+
 try:
     from tqdm import tqdm
 except ImportError:
-    from warnings import warn
-    warn('tqdm not installed. This script takes approximately 5 minutes '
-         'to run. To view live progressbars, please install tqdm. '
-         'Otherwise, be patient.')
+    warn(
+        "tqdm not installed. This script takes approximately 5 minutes "
+        "to run. To view live progressbars, please install tqdm. "
+        "Otherwise, be patient."
+    )
+
     def tqdm(i, **kwargs):
         return i
 
 
-GH_USER = 'scikit-image'
-GH_REPO = 'scikit-image'
-GH_TOKEN = os.environ.get('GH_TOKEN')
+GH_USER = "scikit-image"
+GH_REPO = "scikit-image"
+GH_TOKEN = os.environ.get("GH_TOKEN")
 if GH_TOKEN is None:
     raise RuntimeError(
         "It is necessary that the environment variable `GH_TOKEN` "
         "be set to avoid running into problems with rate limiting. "
         "One can be acquired at https://github.com/settings/tokens.\n\n"
         "You do not need to select any permission boxes while generating "
-        "the token.")
+        "the token."
+    )
 
 g = Github(GH_TOKEN)
-repository = g.get_repo(f'{GH_USER}/{GH_REPO}')
+repository = g.get_repo(f"{GH_USER}/{GH_REPO}")
 
 
 parser = argparse.ArgumentParser(usage=__doc__)
-parser.add_argument('from_commit', help='The starting tag.')
-parser.add_argument('to_commit', help='The head branch.')
-parser.add_argument('--version', help="Version you're about to release.",
-                    default='0.15.0')
+parser.add_argument("from_commit", help="The starting tag.")
+parser.add_argument("to_commit", help="The head branch.")
+parser.add_argument(
+    "--version", help="Version you're about to release.", default="0.15.0"
+)
 
 args = parser.parse_args()
 
@@ -76,26 +79,30 @@ for tag in repository.get_tags():
         previous_tag = tag
         break
 else:
-    raise RuntimeError(f'Desired tag ({args.from_commit}) not found')
+    raise RuntimeError(f"Desired tag ({args.from_commit}) not found")
 
 # For some reason, go get the github commit from the commit to get
 # the correct date
-github_commit =  previous_tag.commit.commit
-previous_tag_date = datetime.strptime(github_commit.last_modified,
-                                      '%a, %d %b %Y %H:%M:%S %Z')
+github_commit = previous_tag.commit.commit
+previous_tag_date = datetime.strptime(
+    github_commit.last_modified, "%a, %d %b %Y %H:%M:%S %Z"
+)
 
 
-all_commits = list(tqdm(repository.get_commits(sha=args.to_commit,
-                                               since=previous_tag_date),
-                        desc=f'Getting all commits between {args.from_commit} '
-                             f'and {args.to_commit}'))
-all_hashes = set(c.sha for c in all_commits)
+all_commits = list(
+    tqdm(
+        repository.get_commits(sha=args.to_commit, since=previous_tag_date),
+        desc=f"Getting all commits between {args.from_commit} " f"and {args.to_commit}",
+    )
+)
+all_hashes = {c.sha for c in all_commits}
 
 authors = set()
 reviewers = set()
 committers = set()
 users = dict()  # keep track of known usernames
 
+
 def find_author_info(commit):
     """Return committer and author of a commit.
 
@@ -114,9 +121,9 @@ def find_author_info(commit):
     committer = None
     if commit.committer is not None:
         committer = commit.committer.name or commit.committer.login
-    git_author = commit.raw_data['commit']['author']['name']
+    git_author = commit.raw_data["commit"]["author"]["name"]
     if commit.author is not None:
-        author = commit.author.name or commit.author.login + f' ({git_author})'
+        author = commit.author.name or commit.author.login + f" ({git_author})"
     else:
         # Users that deleted their accounts will appear as None
         author = git_author
@@ -129,7 +136,8 @@ def add_to_users(users, new_user):
     else:
         users[new_user.login] = new_user.name
 
-for commit in tqdm(all_commits, desc='Getting commiters and authors'):
+
+for commit in tqdm(all_commits, desc="Getting committers and authors"):
     committer, author = find_author_info(commit)
     if committer is not None:
         committers.add(committer)
@@ -141,24 +149,28 @@ for commit in tqdm(all_commits, desc='Getting commiters and authors'):
         add_to_users(users, commit.author)
     authors.add(author)
 
-# this gets found as a commiter
-committers.discard('GitHub Web Flow')
-authors.discard('Azure Pipelines Bot')
+# this gets found as a committer
+committers.discard("GitHub Web Flow")
+authors.discard("Azure Pipelines Bot")
 
-highlights = OrderedDict()
+highlights = {}
 
-highlights['New Feature'] = {}
-highlights['Improvement'] = {}
-highlights['Bugfix'] = {}
-highlights['API Change'] = {}
-highlights['Deprecations'] = {}
-highlights['Build Tool'] = {}
+highlights["New Feature"] = {}
+highlights["Improvement"] = {}
+highlights["Bugfix"] = {}
+highlights["API Change"] = {}
+highlights["Deprecations"] = {}
+highlights["Build Tool"] = {}
 other_pull_requests = {}
 
-for pull in tqdm(g.search_issues(f'repo:{GH_USER}/{GH_REPO} '
-                                 f'merged:>{previous_tag_date.isoformat()} '
-                                 'sort:created-asc'),
-                 desc='Pull Requests...'):
+for pull in tqdm(
+    g.search_issues(
+        f"repo:{GH_USER}/{GH_REPO} "
+        f"merged:>{previous_tag_date.isoformat()} "
+        "sort:created-asc"
+    ),
+    desc="Pull Requests...",
+):
     pr = repository.get_pull(pull.number)
     if pr.merge_commit_sha in all_hashes:
         summary = pull.title
@@ -167,36 +179,33 @@ for pull in tqdm(g.search_issues(f'repo:{GH_USER}/{GH_REPO} '
                 users[review.user.login] = review.user.name
             reviewers.add(users[review.user.login])
         for key, key_dict in highlights.items():
-            pr_title_prefix = (key + ': ').lower()
+            pr_title_prefix = (key + ": ").lower()
             if summary.lower().startswith(pr_title_prefix):
-                key_dict[pull.number] = {
-                    'summary': summary[len(pr_title_prefix):]
-                }
+                key_dict[pull.number] = {"summary": summary[len(pr_title_prefix) :]}
                 break
         else:
-            other_pull_requests[pull.number] = {
-                'summary': summary
-            }
+            other_pull_requests[pull.number] = {"summary": summary}
 
 
 # add Other PRs to the ordered dict to make doc generation easier.
-highlights['Other Pull Request'] = other_pull_requests
+highlights["Other Pull Request"] = other_pull_requests
 
 
 # Now generate the release notes
-announcement_title = f'Announcement: scikit-image {args.version}'
+announcement_title = f"scikit-image {args.version} release notes"
 print(announcement_title)
-print('=' * len(announcement_title))
-
-print(f'''
-We're happy to announce the release of scikit-image v{args.version}!
+print("=" * len(announcement_title))
 
+print(
+    """
 scikit-image is an image processing toolbox for SciPy that includes algorithms
 for segmentation, geometric transformations, color space manipulation,
 analysis, filtering, morphology, feature detection, and more.
-''')
+"""
+)
 
-print("""
+print(
+    """
 For more information, examples, and documentation, please visit our website:
 
 https://scikit-image.org
@@ -208,27 +217,30 @@ for section, pull_request_dicts in highlights.items():
     if not pull_request_dicts:
         continue
     print(f'{section}s\n{"*" * (len(section)+1)}')
-    for number, pull_request_info in pull_request_dicts.items():
-        print(f'- {pull_request_info["summary"]} (#{number})')
+    for pr_num, pr_info in pull_request_dicts.items():
+        print(
+            f"- {pr_info['summary'].strip()}\n  (`#{pr_num} <https://github.com/scikit-image/scikit-image/pull/{pr_num}>`_)."
+        )
 
+contributors = {}
 
-contributors = OrderedDict()
-
-contributors['authors'] = authors
+contributors["authors"] = authors
 # contributors['committers'] = committers
-contributors['reviewers'] = reviewers
+contributors["reviewers"] = reviewers
 
 for section_name, contributor_set in contributors.items():
     print()
-    committer_str = (f'{len(contributor_set)} {section_name} added to this '
-                     'release [alphabetical by first name or login]')
+    committer_str = (
+        f"{len(contributor_set)} {section_name} added to this "
+        "release [alphabetical by first name or login]"
+    )
     print(committer_str)
-    print('-' * len(committer_str))
+    print("-" * len(committer_str))
 
     # Remove None from contributor set if it's in there.
     if None in contributor_set:
         contributor_set.remove(None)
 
     for c in sorted(contributor_set, key=str.lower):
-        print(f'- {c}')
+        print(f"- {c}")
     print()
